@@ -1,9 +1,8 @@
 package Mail::TLSRPT::Failure;
 # ABSTRACT: TLSRPT failure object
-our $VERSION = '1.20200306.1'; # VERSION
+our $VERSION = '1.20200413.1'; # VERSION
 use 5.20.0;
 use Moo;
-use Carp;
 use Mail::TLSRPT::Pragmas;
 use Net::IP;
     has result_type => (is => 'rw', isa => Enum[ qw( starttls-not-supported certificate-host-mismatch certificate-expired certificate-not-trusted validation-failure tlsa-invalid dnssec-invalid dane-required sts-policy-fetch-error sts-policy-invalid sts-webpki-invalid ) ], required => 1);
@@ -21,6 +20,7 @@ sub _coerce_ip {
     return $ip;
 }
 
+
 sub new_from_data($class,$data) {
     my $self = $class->new(
         result_type => $data->{'result-type'},
@@ -35,6 +35,7 @@ sub new_from_data($class,$data) {
     return $self;
 }
 
+
 sub as_struct($self) {
     return {
         'result-type' => $self->result_type,
@@ -48,6 +49,7 @@ sub as_struct($self) {
     };
 }
 
+
 sub as_string($self) {
     my $receiving_ip = $self->receiving_ip ? ' ('.$self->receiving_ip->ip.')' : '';
     return join( "\n",
@@ -62,6 +64,26 @@ sub as_string($self) {
     );
 }
 
+sub _register_prometheus($self,$prometheus) {
+    $prometheus->declare('tlsrpt_failures_total', help=>'TLSRPT failures', type=>'counter' );
+}
+
+
+sub process_prometheus($self,$policy,$report,$prometheus) {
+    $self->_register_prometheus($prometheus);
+    $prometheus->add('tlsrpt_failures_total',$self->failed_session_count,{
+        organization_name=>$report->organization_name,
+        policy_type=>$policy->policy_type,
+        policy_domain=>$policy->policy_domain,
+        policy_mx_host=>$policy->policy_mx_host,
+        result_type=>$self->result_type,
+        sending_mta_ip=>$self->sending_mta_ip,
+        receiving_mx_hostname=>$self->receiving_mx_hostname,
+        receiving_mx_helo=>$self->receiving_mx_helo // '',
+        receiving_ip=>($self->receiving_ip?$self->receiving_ip->ip:''),
+    });
+}
+
 sub _csv_headers($self) {
     return (
         'result type',
@@ -102,7 +124,48 @@ Mail::TLSRPT::Failure - TLSRPT failure object
 
 =head1 VERSION
 
-version 1.20200306.1
+version 1.20200413.1
+
+=head1 SYNOPSIS
+
+my $failure = Mail::TLSRPT::Failure->new(
+    result_type => 'certificate-expired',
+    sending_mta_ip => Net::IP->new($ip),
+    receiving_mx_hostname => 'mx.example.com',
+    receiving_mx_helo => 'mx1.example.com',
+    receiving_ip => Net::IP->new($ip),
+    failed_session_count => 10,
+    additional_information => 'Foo',
+    failure_reason_code => 'Bar',
+);
+
+=head1 DESCRIPTION
+
+Classes to process tlsrpt failure in a report
+
+=head1 CONSTRUCTOR
+
+=head2 I<new($class)>
+
+Create a new object
+
+=head2 I<new_from_data($data)>
+
+Create a new object using a data structure, this will create sub-objects as required.
+
+=head1 METHODS
+
+=head2 I<as_struct>
+
+Return the current object as a data structure
+
+=head2 I<as_string>
+
+Return a textual human readable representation of the current object and its sub-objects
+
+=head2 I<process_prometheus($prometheus,$report)>
+
+Generate metrics using the given Prometheus::Tiny object
 
 =head1 AUTHOR
 
