@@ -1,12 +1,13 @@
 package Mail::TLSRPT::Pragmas;
 # ABSTRACT: Setup system wide pragmas
-our $VERSION = '1.20200306.1'; # VERSION
+our $VERSION = '1.20200413.1'; # VERSION
 use 5.20.0;
 use strict;
 use warnings;
 require feature;
 use Import::Into;
 
+use Carp;
 use English;
 use JSON;
 use Types::Standard;
@@ -14,12 +15,14 @@ use Type::Utils;
 
 use open ':std', ':encoding(UTF-8)';
 
+
 sub import {
   strict->import;
   warnings->import;
   feature->import($_) for ( qw{ postderef signatures } );
   warnings->unimport($_) for ( qw{ experimental::postderef experimental::signatures } );
 
+  Carp->import::into(scalar caller);
   Types::Standard->import::into(scalar caller, qw{ Str Int HashRef ArrayRef Enum } );
   Type::Utils->import::into(scalar caller, qw{ class_type } );
   English->import::into(scalar caller);
@@ -40,7 +43,21 @@ Mail::TLSRPT::Pragmas - Setup system wide pragmas
 
 =head1 VERSION
 
-version 1.20200306.1
+version 1.20200413.1
+
+=head1 SYNOPSIS
+
+Included in all other modules to setup common pragmas and imports
+
+=head1 DESCRIPTION
+
+Setup system wide pragmas
+
+=head1 METHODS
+
+=head2 I<import()>
+
+Import standard pragmas and imports into current namespace
 
 =head1 AUTHOR
 
